@@ -8,10 +8,10 @@
 
 #include "gibber-xmpp-stanza.h"
 
-#ifdef ENABLE_DEBUG
-
 G_BEGIN_DECLS
 
+#ifdef ENABLE_DEBUG
+
 typedef enum
 {
   DEBUG_TRANSPORT         = 1 << 0,
@@ -24,11 +24,12 @@ typedef enum
   DEBUG_RMULTICAST_SENDER = 1 << 7,
   DEBUG_MUC_CONNECTION    = 1 << 8,
   DEBUG_BYTESTREAM        = 1 << 9,
+  DEBUG_FILE_TRANSFER     = 1 << 10,
 } DebugFlags;
 
 #define DEBUG_XMPP (DEBUG_XMPP_READER | DEBUG_XMPP_WRITER)
 
-void gibber_debug_set_flags_from_env ();
+void gibber_debug_set_flags_from_env (void);
 void gibber_debug_set_flags (DebugFlags flags);
 gboolean gibber_debug_flag_is_set (DebugFlags flag);
 void gibber_debug (DebugFlags flag, const gchar *format, ...)
@@ -37,28 +38,6 @@ void gibber_debug_stanza (DebugFlags flag, GibberXmppStanza *stanza,
     const gchar *format, ...)
     G_GNUC_PRINTF (3, 4);
 
-#define gibber_goto_if_reached(label) G_STMT_START{ \
-    g_log (G_LOG_DOMAIN, \
-        G_LOG_LEVEL_CRITICAL, \
-        "file %s: line %d: should not be reached", \
-        __FILE__, \
-        __LINE__); \
-    goto label; }G_STMT_END;
-
-#define gibber_goto_if_fail(expr, label)  G_STMT_START{ \
-    if (expr) {} \
-    else \
-      { \
-        g_log (G_LOG_DOMAIN, \
-            G_LOG_LEVEL_CRITICAL, \
-            "file %s: line %d: assertion `%s' failed", \
-            __FILE__, \
-            __LINE__, \
-            #expr); \
-        goto label; \
-      }; }G_STMT_END
-
-
 #ifdef DEBUG_FLAG
 
 #define DEBUG(format, ...) \
@@ -76,13 +55,16 @@ void gibber_debug_stanza (DebugFlags flag, GibberXmppStanza *stanza,
 
 #ifdef DEBUG_FLAG
 
-#define DEBUG(format, ...)
+#define DEBUG(format, ...) \
+  G_STMT_START { } G_STMT_END
 
-#define DEBUG_STANZA(stanza, format, ...)
+#define DEBUG_STANZA(stanza, format, ...) \
+  G_STMT_START { } G_STMT_END
 
 #define DEBUGGING 0
 
-#define NODE_DEBUG(n, s)
+#define NODE_DEBUG(n, s) \
+  G_STMT_START { } G_STMT_END
 
 #endif /* DEBUG_FLAG */
 
@@ -91,4 +73,3 @@ void gibber_debug_stanza (DebugFlags flag, GibberXmppStanza *stanza,
 G_END_DECLS
 
 #endif
-
