@@ -122,9 +122,6 @@ gibber_oob_file_transfer_finalize (GObject *object)
       g_object_unref (G_OBJECT (self->priv->server));
     }
 
-  if (self->priv->msg)
-    g_object_unref (G_OBJECT (self->priv->msg));
-
   if (self->priv->session != NULL)
     g_object_unref (self->priv->session);
 
@@ -158,8 +155,11 @@ gibber_oob_file_transfer_is_file_offer (GibberXmppStanza *stanza)
     return FALSE;
 
   url = gibber_xmpp_node_get_child (query, "url");
+  if (url == NULL)
+    return FALSE;
+
   url_content = url->content;
-  if (url == NULL || url_content == NULL || strcmp (url_content, "") == 0)
+  if (url_content == NULL || strcmp (url_content, "") == 0)
     return FALSE;
 
   if (url_content[0] == '\n')
@@ -420,12 +420,12 @@ create_transfer_offer (GibberOobFileTransfer *self,
                        GError **error)
 {
   GibberXmppConnection *connection;
-  GibberFdTransport *transport;
 
   /* local host name */
   gchar host_name[NI_MAXHOST];
-  struct sockaddr name_addr;
+  struct sockaddr_storage name_addr;
   socklen_t name_addr_len = sizeof (name_addr);
+  gchar *host_escaped;
 
   GibberXmppStanza *stanza;
   GibberXmppNode *query_node;
@@ -439,25 +439,36 @@ create_transfer_offer (GibberOobFileTransfer *self,
   guint64 size;
 
   g_object_get (GIBBER_FILE_TRANSFER (self), "connection", &connection, NULL);
-  transport = GIBBER_FD_TRANSPORT (connection->transport);
-  if (transport == NULL)
+  if (connection->transport == NULL)
     {
       g_set_error (error, GIBBER_FILE_TRANSFER_ERROR,
           GIBBER_FILE_TRANSFER_ERROR_NOT_CONNECTED, "Null transport");
       return NULL;
     }
 
-  getsockname (transport->fd, &name_addr, &name_addr_len);
+  gibber_transport_get_sockaddr (connection->transport, &name_addr,
+      &name_addr_len);
   g_object_unref (connection);
-  getnameinfo (&name_addr, name_addr_len, host_name, sizeof (host_name), NULL,
-      0, NI_NUMERICHOST);
+  getnameinfo ((struct sockaddr *) &name_addr, name_addr_len, host_name,
+      sizeof (host_name), NULL, 0, NI_NUMERICHOST);
+
+  if (name_addr.ss_family == AF_INET6)
+    {
+      /* put brackets around the IP6 */
+      host_escaped = g_strdup_printf ("[%s]", host_name);
+    }
+  else
+    {
+      /* IPv4: No need to modify the host_name */
+      host_escaped = g_strdup (host_name);
+    }
 
   filename_escaped = g_uri_escape_string (GIBBER_FILE_TRANSFER (self)->filename,
       NULL, FALSE);
-  /* FIXME: this won't work with IPv6 address */
-  url = g_strdup_printf ("http://%s:%d/%s/%s", host_name,
+  url = g_strdup_printf ("http://%s:%d/%s/%s", host_escaped,
       soup_server_get_port (self->priv->server),
       GIBBER_FILE_TRANSFER (self)->id, filename_escaped);
+  g_free (host_escaped);
   g_free (filename_escaped);
   served_name = g_strdup_printf ("/%s/%s", GIBBER_FILE_TRANSFER (self)->id,
       GIBBER_FILE_TRANSFER (self)->filename);
@@ -472,7 +483,7 @@ create_transfer_offer (GibberOobFileTransfer *self,
       GIBBER_FILE_TRANSFER (self)->peer_id);
 
   query_node = gibber_xmpp_node_add_child_ns (stanza->node, "query",
-      GIBBER_XMPP_NS_OOB);
+      GIBBER_XMPP_NS_IQ_OOB);
 
   url_node = gibber_xmpp_node_add_child_with_content (query_node, "url", url);
   gibber_xmpp_node_set_attribute (url_node, "type", "file");
@@ -567,6 +578,8 @@ http_server_cb (SoupServer *server,
   const SoupURI *uri = soup_message_get_uri (msg);
   GibberOobFileTransfer *self = user_data;
   const gchar *accept_encoding;
+  guint64 size;
+  gchar *size_str;
 
   if (msg->method != SOUP_METHOD_GET)
     {
@@ -590,7 +603,13 @@ http_server_cb (SoupServer *server,
   soup_message_headers_append (msg->response_headers, "Content-Type",
       GIBBER_FILE_TRANSFER (self)->content_type);
 
-  self->priv->msg = g_object_ref (msg);
+  size = gibber_file_transfer_get_size (GIBBER_FILE_TRANSFER (self));
+  size_str = g_strdup_printf ("%" G_GUINT64_FORMAT, size);
+  soup_message_headers_append (msg->response_headers, "Content-Length",
+      size_str);
+  g_free (size_str);
+
+  self->priv->msg = msg;
 
   /* iChat accepts only AppleSingle encoding, i.e. file's contents and
    * attributes are stored in the same stream */
@@ -598,7 +617,6 @@ http_server_cb (SoupServer *server,
       "Accept-Encoding");
   if (accept_encoding != NULL && strcmp (accept_encoding, "AppleSingle") == 0)
     {
-      guint64 size;
       guint32 uint32;
       guint16 uint16;
       GByteArray *array;
@@ -607,7 +625,6 @@ http_server_cb (SoupServer *server,
 
       DEBUG ("Using AppleSingle encoding");
 
-      size = gibber_file_transfer_get_size (GIBBER_FILE_TRANSFER (self));
       array = g_byte_array_sized_new (38);
       /* magic number */
       uint32 = htonl (0x51600);
@@ -660,7 +677,46 @@ gibber_oob_file_transfer_offer (GibberFileTransfer *ft)
   /* FIXME we should have only a single server */
   if (self->priv->server == NULL)
     {
-      self->priv->server = soup_server_new (NULL, NULL);
+      /* FIXME: libsoup can't listen on IPv4 and IPv6 interfaces at the same
+       * time. http://bugzilla.gnome.org/show_bug.cgi?id=522519
+       * We have to check which IP will be send when creating the stanza. */
+        GibberXmppConnection *connection;
+        struct sockaddr_storage name_addr;
+        socklen_t name_addr_len = sizeof (name_addr);
+
+        g_object_get (GIBBER_FILE_TRANSFER (self), "connection", &connection,
+            NULL);
+        if (connection->transport == NULL)
+          {
+            g_set_error (&error, GIBBER_FILE_TRANSFER_ERROR,
+                GIBBER_FILE_TRANSFER_ERROR_NOT_CONNECTED, "Null transport");
+            gibber_file_transfer_emit_error (GIBBER_FILE_TRANSFER (self),
+                error);
+            g_error_free (error);
+            return;
+          }
+
+        gibber_transport_get_peeraddr (connection->transport, &name_addr,
+            &name_addr_len);
+        g_object_unref (connection);
+
+        if (name_addr.ss_family == AF_INET6)
+          {
+            /* IPv6 server */
+            SoupAddress *addr;
+
+            addr = soup_address_new_any (SOUP_ADDRESS_FAMILY_IPV6, 0);
+            self->priv->server = soup_server_new (SOUP_SERVER_INTERFACE,
+                addr, NULL);
+
+            g_object_unref (addr);
+          }
+        else
+          {
+            /* IPv4 server */
+            self->priv->server = soup_server_new (NULL, NULL);
+          }
+
       soup_server_run_async (self->priv->server);
     }
 
@@ -742,7 +798,7 @@ gibber_oob_file_transfer_cancel (GibberFileTransfer *ft,
   gibber_xmpp_node_set_attribute (stanza->node, "id", ft->id);
 
   query = gibber_xmpp_node_add_child_ns (stanza->node, "query",
-      GIBBER_XMPP_NS_OOB);
+      GIBBER_XMPP_NS_IQ_OOB);
   gibber_xmpp_node_add_child_with_content (query, "url", self->priv->url);
 
   error_node = gibber_xmpp_node_add_child (stanza->node, "error");
