@@ -24,17 +24,6 @@
 
 G_BEGIN_DECLS
 
-GQuark gibber_xmpp_connection_listener_error_quark (void);
-#define GIBBER_XMPP_CONNECTION_LISTENER_ERROR \
-  gibber_xmpp_connection_listener_error_quark ()
-
-typedef enum
-{
-  GIBBER_XMPP_CONNECTION_LISTENER_ERROR_ALREADY_LISTENING,
-  GIBBER_XMPP_CONNECTION_LISTENER_ERROR_ADDR_IN_USE,
-  GIBBER_XMPP_CONNECTION_LISTENER_ERROR_FAILED,
-} GibberXmppConnectionListenerError;
-
 typedef struct _GibberXmppConnectionListener GibberXmppConnectionListener;
 typedef struct _GibberXmppConnectionListenerClass
     GibberXmppConnectionListenerClass;
@@ -68,13 +57,18 @@ GType gibber_xmpp_connection_listener_get_type (void);
   (G_TYPE_INSTANCE_GET_CLASS ((obj), GIBBER_TYPE_XMPP_CONNECTION_LISTENER,\
                               GibberXmppConnectionListenerClass))
 
+GQuark gibber_xmpp_connection_listener_error_quark (void);
+
 GibberXmppConnectionListener *
 gibber_xmpp_connection_listener_new (void);
 
-gboolean
-gibber_xmpp_connection_listener_listen (
+gboolean gibber_xmpp_connection_listener_listen (
     GibberXmppConnectionListener *listener, int port, GError **error);
 
+int gibber_xmpp_connection_listener_get_port (
+    GibberXmppConnectionListener *listener);
+
+
 G_END_DECLS
 
 #endif /* #ifndef __GIBBER_XMPP_CONNECTION_LISTENER_H__ */
