@@ -43,6 +43,7 @@ typedef struct _SalutDisco SalutDisco;
 
 typedef struct _SalutConnection SalutConnection;
 typedef struct _SalutConnectionClass SalutConnectionClass;
+typedef struct _SalutConnectionPrivate SalutConnectionPrivate;
 
 struct _SalutConnectionClass {
   TpBaseConnectionClass parent_class;
@@ -62,7 +63,7 @@ struct _SalutConnection {
   /* Our name on the network */
   gchar *name;
 
-  gpointer priv;
+  SalutConnectionPrivate *priv;
 };
 
 GType salut_connection_get_type (void);
@@ -100,6 +101,10 @@ salut_connection_olpc_observe_muc_stanza (SalutConnection *self, TpHandle room,
     TpHandle sender, GibberXmppStanza *stanza);
 #endif
 
+const gchar * const *salut_connection_get_implemented_interfaces (void);
+
+gchar *salut_normalize_non_empty (const gchar *id, GError **error);
+
 G_END_DECLS
 
 #endif /* #ifndef __SALUT_CONNECTION_H__*/
