@@ -27,30 +27,31 @@
 #include "salut-connection.h"
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/enums.h>
+#include <telepathy-glib/gtypes.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/channel-iface.h>
 #include <telepathy-glib/svc-channel.h>
+#include <telepathy-glib/svc-generic.h>
 
 #define DEBUG_FLAG DEBUG_ROOMLIST
 #include "debug.h"
 
-#define SALUT_TP_TYPE_ROOM_STRUCT (dbus_g_type_get_struct ("GValueArray", \
-      G_TYPE_UINT, \
-      G_TYPE_STRING, \
-      dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), \
-      G_TYPE_INVALID))
-
 static void channel_iface_init (gpointer, gpointer);
 static void roomlist_iface_init (gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE (SalutRoomlistChannel, salut_roomlist_channel,
     G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
+      tp_dbus_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_ROOM_LIST,
       roomlist_iface_init);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_EXPORTABLE_CHANNEL, NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_IFACE, NULL)
     );
 
+static const char *salut_roomlist_channel_interfaces[] = { NULL };
+
 /* properties */
 enum
 {
@@ -59,6 +60,14 @@ enum
   PROP_HANDLE_TYPE,
   PROP_HANDLE,
   PROP_CONNECTION,
+  PROP_INTERFACES,
+  PROP_CONFERENCE_SERVER,
+  PROP_TARGET_ID,
+  PROP_INITIATOR_HANDLE,
+  PROP_INITIATOR_ID,
+  PROP_REQUESTED,
+  PROP_CHANNEL_DESTROYED,
+  PROP_CHANNEL_PROPERTIES,
   LAST_PROPERTY
 };
 
@@ -77,7 +86,7 @@ struct _SalutRoomlistChannelPrivate
 };
 
 #define SALUT_ROOMLIST_CHANNEL_GET_PRIVATE(obj) \
-    ((SalutRoomlistChannelPrivate *) ((SalutRoomlistChannel *)obj)->priv)
+    ((SalutRoomlistChannelPrivate *) ((SalutRoomlistChannel *) obj)->priv)
 
 static void
 salut_roomlist_channel_init (SalutRoomlistChannel *self)
@@ -118,6 +127,7 @@ salut_roomlist_channel_get_property (GObject *object,
   SalutRoomlistChannel *chan = SALUT_ROOMLIST_CHANNEL (object);
   SalutRoomlistChannelPrivate *priv =
     SALUT_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
+  TpBaseConnection *conn = (TpBaseConnection *) priv->connection;
 
   switch (property_id) {
     case PROP_OBJECT_PATH:
@@ -135,6 +145,49 @@ salut_roomlist_channel_get_property (GObject *object,
     case PROP_CONNECTION:
       g_value_set_object (value, priv->connection);
       break;
+    case PROP_INTERFACES:
+      g_value_set_static_boxed (value, salut_roomlist_channel_interfaces);
+      break;
+    case PROP_CONFERENCE_SERVER:
+      /* Salut does not use a server, so this string is always empty */
+      g_value_set_string (value, "");
+      break;
+    case PROP_TARGET_ID:
+      g_value_set_static_string (value, "");
+      break;
+    case PROP_INITIATOR_HANDLE:
+      /* Room listing is always initiated by the local user */
+      g_value_set_uint (value, conn->self_handle);
+      break;
+    case PROP_INITIATOR_ID:
+        {
+          TpHandleRepoIface *repo = tp_base_connection_get_handles (conn,
+              TP_HANDLE_TYPE_CONTACT);
+
+          g_value_set_string (value,
+              tp_handle_inspect (repo, conn->self_handle));
+        }
+      break;
+    case PROP_REQUESTED:
+      g_value_set_boolean (value, TRUE);
+      break;
+    case PROP_CHANNEL_DESTROYED:
+      g_value_set_boolean (value, priv->closed);
+      break;
+    case PROP_CHANNEL_PROPERTIES:
+      g_value_take_boxed (value,
+          tp_dbus_properties_mixin_make_properties_hash (object,
+              TP_IFACE_CHANNEL, "TargetHandle",
+              TP_IFACE_CHANNEL, "TargetHandleType",
+              TP_IFACE_CHANNEL, "ChannelType",
+              TP_IFACE_CHANNEL, "TargetID",
+              TP_IFACE_CHANNEL, "InitiatorHandle",
+              TP_IFACE_CHANNEL, "InitiatorID",
+              TP_IFACE_CHANNEL, "Requested",
+              TP_IFACE_CHANNEL, "Interfaces",
+              TP_IFACE_CHANNEL_TYPE_ROOM_LIST, "Server",
+              NULL));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -193,6 +246,34 @@ salut_roomlist_channel_class_init (
 {
   GObjectClass *object_class = G_OBJECT_CLASS (salut_roomlist_channel_class);
   GParamSpec *param_spec;
+  static TpDBusPropertiesMixinPropImpl channel_props[] = {
+      { "TargetHandleType", "handle-type", NULL },
+      { "TargetHandle", "handle", NULL },
+      { "TargetID", "target-id", NULL },
+      { "InitiatorHandle", "initiator-handle", NULL },
+      { "InitiatorID", "initiator-id", NULL },
+      { "Requested", "requested", NULL },
+      { "ChannelType", "channel-type", NULL },
+      { "Interfaces", "interfaces", NULL },
+      { NULL }
+  };
+  static TpDBusPropertiesMixinPropImpl roomlist_props[] = {
+      { "Server", "conference-server", NULL },
+      { NULL }
+  };
+  static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
+      { TP_IFACE_CHANNEL,
+        tp_dbus_properties_mixin_getter_gobject_properties,
+        NULL,
+        channel_props,
+      },
+      { TP_IFACE_CHANNEL_TYPE_ROOM_LIST,
+        tp_dbus_properties_mixin_getter_gobject_properties,
+        NULL,
+        roomlist_props,
+      },
+      { NULL }
+  };
 
   g_type_class_add_private (salut_roomlist_channel_class,
       sizeof (SalutRoomlistChannelPrivate));
@@ -213,6 +294,42 @@ salut_roomlist_channel_class_init (
       "handle-type");
   g_object_class_override_property (object_class, PROP_HANDLE,
       "handle");
+  g_object_class_override_property (object_class, PROP_CHANNEL_DESTROYED,
+      "channel-destroyed");
+  g_object_class_override_property (object_class, PROP_CHANNEL_PROPERTIES,
+      "channel-properties");
+
+  param_spec = g_param_spec_boxed ("interfaces", "Extra D-Bus interfaces",
+      "Additional Channel.Interface.* interfaces",
+      G_TYPE_STRV,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INTERFACES, param_spec);
+
+  param_spec = g_param_spec_string ("target-id", "Target JID",
+      "The string obtained by inspecting this channel's handle",
+      NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_TARGET_ID, param_spec);
+
+  param_spec = g_param_spec_uint ("initiator-handle", "Initiator's handle",
+      "The contact who initiated the channel",
+      0, G_MAXUINT32, 0,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INITIATOR_HANDLE,
+      param_spec);
+
+  param_spec = g_param_spec_string ("initiator-id", "Initiator's bare JID",
+      "The string obtained by inspecting the initiator-handle",
+      NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INITIATOR_ID,
+      param_spec);
+
+  param_spec = g_param_spec_boolean ("requested", "Requested?",
+      "True if this channel was requested by the local user",
+      FALSE,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_REQUESTED, param_spec);
 
   param_spec = g_param_spec_object ("connection", "SalutConnection object",
                                     "Salut connection object that owns this "
@@ -220,9 +337,21 @@ salut_roomlist_channel_class_init (
                                     SALUT_TYPE_CONNECTION,
                                     G_PARAM_CONSTRUCT_ONLY |
                                     G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
+                                    G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
+
+  param_spec = g_param_spec_string ("conference-server",
+      "Name of conference server to use",
+      "Name of conference server to use, which is an empty string for Salut",
+      "",
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_CONFERENCE_SERVER,
+      param_spec);
+
+  salut_roomlist_channel_class->dbus_props_class.interfaces = prop_interfaces;
+  tp_dbus_properties_mixin_class_init (object_class,
+      G_STRUCT_OFFSET (SalutRoomlistChannelClass, dbus_props_class));
+
 }
 
 static void
@@ -244,13 +373,13 @@ rooms_free (SalutRoomlistChannel *self)
       TpHandle handle;
 
       boxed = g_ptr_array_index (priv->rooms, i);
-      g_value_init (&room, SALUT_TP_TYPE_ROOM_STRUCT);
+      g_value_init (&room, TP_STRUCT_TYPE_ROOM_INFO);
       g_value_set_static_boxed (&room, boxed);
       dbus_g_type_struct_get (&room,
           0, &handle,
           G_MAXUINT);
 
-      g_boxed_free (SALUT_TP_TYPE_ROOM_STRUCT, boxed);
+      g_boxed_free (TP_STRUCT_TYPE_ROOM_INFO, boxed);
       tp_handle_unref (room_repo, handle);
     }
 
@@ -272,8 +401,8 @@ salut_roomlist_channel_dispose (GObject *object)
 
   if (!priv->closed)
     {
-      tp_svc_channel_emit_closed ((TpSvcChannel *)object);
       priv->closed = TRUE;
+      tp_svc_channel_emit_closed ((TpSvcChannel *) object);
     }
 
   if (G_OBJECT_CLASS (salut_roomlist_channel_parent_class)->dispose)
@@ -323,6 +452,7 @@ salut_roomlist_channel_add_room (SalutRoomlistChannel *self,
   GValue room = {0,};
   TpHandle handle;
   GHashTable *keys;
+  GValue handle_name = {0,};
 
   handle = tp_handle_ensure (room_repo, room_name, NULL, NULL);
   if (handle == 0)
@@ -330,9 +460,14 @@ salut_roomlist_channel_add_room (SalutRoomlistChannel *self,
 
   keys = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 
-  g_value_init (&room, SALUT_TP_TYPE_ROOM_STRUCT);
+  /* handle-name */
+  g_value_init (&handle_name, G_TYPE_STRING);
+  g_value_take_string (&handle_name, (gchar *) room_name);
+  g_hash_table_insert (keys, "handle-name", &handle_name);
+
+  g_value_init (&room, TP_STRUCT_TYPE_ROOM_INFO);
   g_value_take_boxed (&room,
-      dbus_g_type_specialized_construct (SALUT_TP_TYPE_ROOM_STRUCT));
+      dbus_g_type_specialized_construct (TP_STRUCT_TYPE_ROOM_INFO));
   dbus_g_type_struct_set (&room,
       0, handle,
       1, "org.freedesktop.Telepathy.Channel.Type.Text",
@@ -367,7 +502,7 @@ salut_roomlist_channel_remove_room (SalutRoomlistChannel *self,
       TpHandle h;
 
       boxed = g_ptr_array_index (priv->rooms, i);
-      g_value_init (&room, SALUT_TP_TYPE_ROOM_STRUCT);
+      g_value_init (&room, TP_STRUCT_TYPE_ROOM_INFO);
       g_value_set_static_boxed (&room, boxed);
       dbus_g_type_struct_get (&room,
           0, &h,
@@ -375,7 +510,7 @@ salut_roomlist_channel_remove_room (SalutRoomlistChannel *self,
 
       if (handle == h)
         {
-          g_boxed_free (SALUT_TP_TYPE_ROOM_STRUCT, boxed);
+          g_boxed_free (TP_STRUCT_TYPE_ROOM_INFO, boxed);
           g_ptr_array_remove_index_fast (priv->rooms, i);
           tp_handle_unref (room_repo, handle);
           DEBUG ("remove %s", room_name);
@@ -449,9 +584,8 @@ static void
 salut_roomlist_channel_get_interfaces (TpSvcChannel *iface,
                                        DBusGMethodInvocation *context)
 {
-  const char *interfaces[] = { NULL };
-
-  tp_svc_channel_return_from_get_interfaces (context, interfaces);
+  tp_svc_channel_return_from_get_interfaces (context,
+    salut_roomlist_channel_interfaces);
 }
 
 
@@ -529,7 +663,7 @@ static void
 channel_iface_init (gpointer g_iface,
                     gpointer iface_data)
 {
-  TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
+  TpSvcChannelClass *klass = (TpSvcChannelClass *) g_iface;
 
 #define IMPLEMENT(x) tp_svc_channel_implement_##x (\
     klass, salut_roomlist_channel_##x)
@@ -545,7 +679,7 @@ roomlist_iface_init (gpointer g_iface,
                      gpointer iface_data)
 {
   TpSvcChannelTypeRoomListClass *klass =
-    (TpSvcChannelTypeRoomListClass *)g_iface;
+    (TpSvcChannelTypeRoomListClass *) g_iface;
 
 #define IMPLEMENT(x) tp_svc_channel_type_room_list_implement_##x (\
     klass, salut_roomlist_channel_##x)
