@@ -21,9 +21,16 @@ salut_create_connection_manager (void)
 int
 main (int argc, char **argv)
 {
+  GLogLevelFlags fatal_mask;
+
   g_type_init ();
   g_thread_init (NULL);
 
+  /* treat criticals as, well, critical */
+  fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
+  fatal_mask |= G_LOG_LEVEL_CRITICAL;
+  g_log_set_always_fatal (fatal_mask);
+
 #ifdef ENABLE_DEBUG
   tp_debug_divert_messages (g_getenv ("SALUT_LOGFILE"));
   debug_set_flags_from_env ();
