@@ -76,9 +76,9 @@ G_DEFINE_TYPE_WITH_CODE (SalutTubeStream, salut_tube_stream, G_TYPE_OBJECT,
       tp_dbus_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
     G_IMPLEMENT_INTERFACE (SALUT_TYPE_TUBE_IFACE, tube_iface_init);
-    G_IMPLEMENT_INTERFACE (SALUT_TYPE_SVC_CHANNEL_TYPE_STREAM_TUBE,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_STREAM_TUBE,
       streamtube_iface_init);
-    G_IMPLEMENT_INTERFACE (SALUT_TYPE_SVC_CHANNEL_INTERFACE_TUBE,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_TUBE,
       NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_EXPORTABLE_CHANNEL, NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_IFACE, NULL));
@@ -88,14 +88,14 @@ static const gchar *salut_tube_stream_interfaces[] = {
     /* If more interfaces are added, either keep Tube as the first, or change
      * the implementations of salut_tube_stream_get_interfaces () and
      * salut_tube_stream_get_property () too */
-    SALUT_IFACE_CHANNEL_INTERFACE_TUBE,
+    TP_IFACE_CHANNEL_INTERFACE_TUBE,
     NULL
 };
 
 static const gchar * const salut_tube_stream_channel_allowed_properties[] = {
     TP_IFACE_CHANNEL ".TargetHandle",
     TP_IFACE_CHANNEL ".TargetID",
-    SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
+    TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
     NULL
 };
 
@@ -191,10 +191,14 @@ struct _SalutTubeStreamPrivate
    */
   GHashTable *transport_to_bytestream;
 
+  /* (GibberTransport *) -> guint */
+  GHashTable *transport_to_id;
+  guint last_connection_id;
+
   TpHandle initiator;
   gchar *service;
   GHashTable *parameters;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   /* whether the tube is already offered at construct-time (with the
    * Channel.Type.Tubes interface) */
   gboolean offered;
@@ -277,6 +281,31 @@ transport_handler (GibberTransport *transport,
       (const gchar *) data->data);
 }
 
+static void
+fire_connection_closed (SalutTubeStream *self,
+    GibberTransport *transport,
+    const gchar *error,
+    const gchar *debug_msg)
+{
+  SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
+  guint connection_id;
+
+  connection_id = GPOINTER_TO_UINT (g_hash_table_lookup (priv->transport_to_id,
+        transport));
+  if (connection_id == 0)
+    {
+      DEBUG ("ConnectionClosed has already been fired for this connection");
+      return;
+    }
+
+  /* remove the ID so we are sure we won't fire ConnectionClosed twice for the
+   * same connection. */
+  g_hash_table_remove (priv->transport_to_id, transport);
+
+  tp_svc_channel_type_stream_tube_emit_connection_closed (self,
+      connection_id, error, debug_msg);
+}
+
 static void
 transport_disconnected_cb (GibberTransport *transport,
                            SalutTubeStream *self)
@@ -284,6 +313,9 @@ transport_disconnected_cb (GibberTransport *transport,
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
   GibberBytestreamIface *bytestream;
 
+  fire_connection_closed (self, transport, TP_ERROR_STR_CANCELLED,
+      "local socket has been disconnected");
+
   bytestream = g_hash_table_lookup (priv->transport_to_bytestream, transport);
   if (bytestream == NULL)
     return;
@@ -306,11 +338,15 @@ remove_transport (SalutTubeStream *self,
 
   gibber_transport_disconnect (transport);
 
+  fire_connection_closed (self, transport, TP_ERROR_STR_CONNECTION_LOST,
+      "bytestream has been broken");
+
   /* the transport may not be in transport_to_bytestream if the bytestream was
    * not fully open */
   g_hash_table_remove (priv->transport_to_bytestream, transport);
 
   g_hash_table_remove (priv->bytestream_to_transport, bytestream);
+  g_hash_table_remove (priv->transport_to_id, transport);
 }
 
 static void
@@ -449,6 +485,9 @@ extra_bytestream_negotiate_cb (GibberBytestreamIface *bytestream,
     {
       DEBUG ("initiator refused new bytestream");
 
+      fire_connection_closed (self, data->transport,
+          TP_ERROR_STR_CONNECTION_REFUSED, "connection has been refused");
+
       g_object_unref (data->transport);
       g_slice_free (struct _extra_bytestream_negotiate_cb_data, data);
       return;
@@ -644,6 +683,32 @@ start_stream_direct (SalutTubeStream *self,
   return TRUE;
 }
 
+static guint
+generate_connection_id (SalutTubeStream *self,
+    GibberTransport *transport)
+{
+  SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
+
+  priv->last_connection_id++;
+
+  g_hash_table_insert (priv->transport_to_id, transport,
+      GUINT_TO_POINTER (priv->last_connection_id));
+
+  return priv->last_connection_id;
+}
+
+static void
+fire_new_local_connection (SalutTubeStream *self,
+    GibberTransport *transport)
+{
+  guint connection_id;
+
+  connection_id = generate_connection_id (self, transport);
+
+  tp_svc_channel_type_stream_tube_emit_new_local_connection (self,
+      connection_id);
+}
+
 /* callback for listening connections from the local application */
 static void
 local_new_connection_cb (GibberListener *listener,
@@ -670,6 +735,7 @@ local_new_connection_cb (GibberListener *listener,
       if (!start_stream_direct (self, transport, NULL))
         {
           DEBUG ("closing new client connection");
+          return;
         }
     }
   else
@@ -677,11 +743,38 @@ local_new_connection_cb (GibberListener *listener,
       if (!start_stream_initiation (self, transport, NULL))
         {
           DEBUG ("closing new client connection");
+          return;
         }
     }
+
+  fire_new_local_connection (self, transport);
 }
 
-static gboolean
+static void
+fire_new_remote_connection (SalutTubeStream *self,
+    GibberTransport *transport,
+    TpHandle contact)
+{
+  SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
+  GValue access_control_param = {0,};
+  guint connection_id;
+
+  g_assert (priv->access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST);
+
+  /* set a dummy value */
+  g_value_init (&access_control_param, G_TYPE_INT);
+  g_value_set_int (&access_control_param, 0);
+
+  connection_id = GPOINTER_TO_UINT (g_hash_table_lookup (priv->transport_to_id,
+        transport));
+  g_assert (connection_id != 0);
+
+  tp_svc_channel_type_stream_tube_emit_new_remote_connection (self,
+      contact, &access_control_param, connection_id);
+  g_value_unset (&access_control_param);
+}
+
+static GibberTransport *
 new_connection_to_socket (SalutTubeStream *self,
                           GibberBytestreamIface *bytestream)
 {
@@ -729,6 +822,8 @@ new_connection_to_socket (SalutTubeStream *self,
    * its data. */
   gibber_transport_block_receiving (transport, TRUE);
 
+  generate_connection_id (self, transport);
+
   g_hash_table_insert (priv->bytestream_to_transport, g_object_ref (bytestream),
       g_object_ref (transport));
 
@@ -736,7 +831,7 @@ new_connection_to_socket (SalutTubeStream *self,
       G_CALLBACK (extra_bytestream_state_changed_cb), self);
 
   g_object_unref (transport);
-  return TRUE;
+  return transport;
 }
 
 static gboolean
@@ -790,6 +885,13 @@ tube_stream_open (SalutTubeStream *self,
           g_free (path);
           return FALSE;
         }
+
+      if (priv->access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST)
+        {
+          /* Everyone can use the socket */
+          chmod (path, 0777);
+        }
+
       g_free (path);
     }
   else if (priv->address_type == TP_SOCKET_ADDRESS_TYPE_IPV4)
@@ -862,6 +964,10 @@ salut_tube_stream_init (SalutTubeStream *self)
       g_direct_equal, (GDestroyNotify) g_object_unref,
       (GDestroyNotify) g_object_unref);
 
+  priv->transport_to_id = g_hash_table_new_full (g_direct_hash,
+      g_direct_equal, NULL, NULL);
+  priv->last_connection_id = 0;
+
   priv->address_type = TP_SOCKET_ADDRESS_TYPE_UNIX;
   priv->address = NULL;
   priv->access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
@@ -893,6 +999,8 @@ close_each_extra_bytestream (gpointer key,
 
   gibber_bytestream_iface_close (bytestream, NULL);
   gibber_transport_disconnect (transport);
+  fire_connection_closed (self, transport, TP_ERROR_STR_CANCELLED,
+      "tube is closing");
 
   g_hash_table_remove (priv->transport_to_bytestream, transport);
 
@@ -943,6 +1051,12 @@ salut_tube_stream_dispose (GObject *object)
       priv->bytestream_to_transport = NULL;
     }
 
+  if (priv->transport_to_id != NULL)
+    {
+      g_hash_table_destroy (priv->transport_to_id);
+      priv->transport_to_id = NULL;
+    }
+
   tp_handle_unref (contact_repo, priv->initiator);
 
   if (priv->local_listener != NULL)
@@ -1038,7 +1152,7 @@ salut_tube_stream_get_property (GObject *object,
           }
         break;
       case PROP_CHANNEL_TYPE:
-        g_value_set_static_string (value, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+        g_value_set_static_string (value, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
         break;
       case PROP_HANDLE:
         g_value_set_uint (value, priv->handle);
@@ -1107,8 +1221,8 @@ salut_tube_stream_get_property (GObject *object,
               TP_IFACE_CHANNEL, "InitiatorID",
               TP_IFACE_CHANNEL, "Requested",
               TP_IFACE_CHANNEL, "Interfaces",
-              SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE, "Service",
-              SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE, "SupportedSocketTypes",
+              TP_IFACE_CHANNEL_TYPE_STREAM_TUBE, "Service",
+              TP_IFACE_CHANNEL_TYPE_STREAM_TUBE, "SupportedSocketTypes",
               NULL);
 
           if (priv->initiator != priv->self_handle)
@@ -1119,12 +1233,12 @@ salut_tube_stream_get_property (GObject *object,
               /* FIXME: use tp_dbus_properties_mixin_add_properties once it's
                * added in tp-glib */
               tp_dbus_properties_mixin_get (object,
-                  SALUT_IFACE_CHANNEL_INTERFACE_TUBE, "Parameters",
+                  TP_IFACE_CHANNEL_INTERFACE_TUBE, "Parameters",
                   prop_value, NULL);
               g_assert (G_IS_VALUE (prop_value));
 
               g_hash_table_insert (properties,
-                  g_strdup_printf ("%s.%s", SALUT_IFACE_CHANNEL_INTERFACE_TUBE,
+                  g_strdup_printf ("%s.%s", TP_IFACE_CHANNEL_INTERFACE_TUBE,
                     "Parameters"), prop_value);
             }
 
@@ -1364,13 +1478,13 @@ salut_tube_stream_constructor (GType type,
   if (priv->initiator == priv->self_handle)
     {
       /* We initiated this tube */
-      priv->state = SALUT_TUBE_CHANNEL_STATE_NOT_OFFERED;
+      priv->state = TP_TUBE_CHANNEL_STATE_NOT_OFFERED;
       /* FIXME: we should probably remove this offer_needed */
       priv->offer_needed = TRUE;
     }
   else
     {
-      priv->state = SALUT_TUBE_CHANNEL_STATE_LOCAL_PENDING;
+      priv->state = TP_TUBE_CHANNEL_STATE_LOCAL_PENDING;
     }
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
@@ -1423,12 +1537,12 @@ salut_tube_stream_class_init (SalutTubeStreamClass *salut_tube_stream_class)
         NULL,
         channel_props,
       },
-      { SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE,
+      { TP_IFACE_CHANNEL_TYPE_STREAM_TUBE,
         tp_dbus_properties_mixin_getter_gobject_properties,
         NULL,
         stream_tube_props,
       },
-      { SALUT_IFACE_CHANNEL_INTERFACE_TUBE,
+      { TP_IFACE_CHANNEL_INTERFACE_TUBE,
         tp_dbus_properties_mixin_getter_gobject_properties,
         NULL,
         tube_iface_props,
@@ -1776,7 +1890,7 @@ salut_tube_stream_accept (SalutTubeIface *tube,
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
   GibberXmppStanza *reply;
 
-  if (priv->state != SALUT_TUBE_CHANNEL_STATE_LOCAL_PENDING)
+  if (priv->state != TP_TUBE_CHANNEL_STATE_LOCAL_PENDING)
     return TRUE;
 
   if (!tube_stream_open (self, error))
@@ -1800,11 +1914,11 @@ salut_tube_stream_accept (SalutTubeIface *tube,
         }
     }
 
-  priv->state = SALUT_TUBE_CHANNEL_STATE_OPEN;
+  priv->state = TP_TUBE_CHANNEL_STATE_OPEN;
   g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
 
-  salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
-      self, SALUT_TUBE_CHANNEL_STATE_OPEN);
+  tp_svc_channel_interface_tube_emit_tube_channel_state_changed (
+      self, TP_TUBE_CHANNEL_STATE_OPEN);
 
   return TRUE;
 }
@@ -1826,8 +1940,8 @@ salut_tube_stream_accepted (SalutTubeIface *tube)
   priv->state = TP_TUBE_STATE_OPEN;
   g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
 
-  salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
-      self, SALUT_TUBE_CHANNEL_STATE_OPEN);
+  tp_svc_channel_interface_tube_emit_tube_channel_state_changed (
+      self, TP_TUBE_CHANNEL_STATE_OPEN);
 }
 
 /**
@@ -2018,6 +2132,7 @@ salut_tube_stream_add_bytestream (SalutTubeIface *tube,
 {
   SalutTubeStream *self = SALUT_TUBE_STREAM (tube);
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
+  GibberTransport *transport;
 
   if (priv->initiator != priv->self_handle)
     {
@@ -2029,17 +2144,18 @@ salut_tube_stream_add_bytestream (SalutTubeIface *tube,
     }
 
   /* New bytestream, let's connect to the socket */
-  if (new_connection_to_socket (self, bytestream))
+  transport = new_connection_to_socket (self, bytestream);
+  if (transport != NULL)
     {
       TpHandle contact;
       gchar *peer_id;
       TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
           (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
-      if (priv->state == SALUT_TUBE_CHANNEL_STATE_REMOTE_PENDING)
+      if (priv->state == TP_TUBE_CHANNEL_STATE_REMOTE_PENDING)
         {
           DEBUG ("Received first connection. Tube is now open");
-          priv->state = SALUT_TUBE_CHANNEL_STATE_OPEN;
+          priv->state = TP_TUBE_CHANNEL_STATE_OPEN;
           g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
         }
 
@@ -2052,8 +2168,7 @@ salut_tube_stream_add_bytestream (SalutTubeIface *tube,
 
       g_signal_emit (G_OBJECT (self), signals[NEW_CONNECTION], 0, contact);
 
-      salut_svc_channel_type_stream_tube_emit_stream_tube_new_connection (
-          self, contact);
+      fire_new_remote_connection (self, transport, contact);
 
       tp_handle_unref (contact_repo, contact);
       g_free (peer_id);
@@ -2242,25 +2357,24 @@ salut_tube_stream_check_params (TpSocketAddressType address_type,
 }
 
 /**
- * salut_tube_stream_offer_stream_tube
+ * salut_tube_stream_offer_async
  *
- * Implements D-Bus method OfferStreamTube
+ * Implements D-Bus method Offer
  * on org.freedesktop.Telepathy.Channel.Type.StreamTube
  */
 static void
-salut_tube_stream_offer_stream_tube (SalutSvcChannelTypeStreamTube *iface,
-                                     guint address_type,
-                                     const GValue *address,
-                                     guint access_control,
-                                     const GValue *access_control_param,
-                                     GHashTable *parameters,
-                                     DBusGMethodInvocation *context)
+salut_tube_stream_offer_async (TpSvcChannelTypeStreamTube *iface,
+    guint address_type,
+    const GValue *address,
+    guint access_control,
+    GHashTable *parameters,
+    DBusGMethodInvocation *context)
 {
   SalutTubeStream *self = SALUT_TUBE_STREAM (iface);
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
   GError *error = NULL;
 
-  if (priv->state != SALUT_TUBE_CHANNEL_STATE_NOT_OFFERED)
+  if (priv->state != TP_TUBE_CHANNEL_STATE_NOT_OFFERED)
     {
       g_set_error (&error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "Tube is not in the not offered state");
@@ -2270,7 +2384,7 @@ salut_tube_stream_offer_stream_tube (SalutSvcChannelTypeStreamTube *iface,
     }
 
   if (!salut_tube_stream_check_params (address_type, address,
-        access_control, access_control_param, &error))
+        access_control, NULL, &error))
     {
       dbus_g_method_return_error (context, error);
       g_error_free (error);
@@ -2286,7 +2400,6 @@ salut_tube_stream_offer_stream_tube (SalutSvcChannelTypeStreamTube *iface,
   g_assert (priv->access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST);
   priv->access_control = access_control;
   g_assert (priv->access_control_param == NULL);
-  priv->access_control_param = tp_g_value_slice_dup (access_control_param);
 
   g_object_set (self, "parameters", parameters, NULL);
 
@@ -2297,27 +2410,27 @@ salut_tube_stream_offer_stream_tube (SalutSvcChannelTypeStreamTube *iface,
       return;
     }
 
-  salut_svc_channel_type_stream_tube_return_from_offer_stream_tube (context);
+  tp_svc_channel_type_stream_tube_return_from_offer (context);
 }
 
 /**
- * salut_tube_stream_accept_stream_tube
+ * salut_tube_stream_accept_async
  *
- * Implements D-Bus method AcceptStreamTube
+ * Implements D-Bus method Accept
  * on org.freedesktop.Telepathy.Channel.Type.StreamTube
  */
 static void
-salut_tube_stream_accept_stream_tube (SalutSvcChannelTypeStreamTube *iface,
-                                      guint address_type,
-                                      guint access_control,
-                                      const GValue *access_control_param,
-                                      DBusGMethodInvocation *context)
+salut_tube_stream_accept_async (TpSvcChannelTypeStreamTube *iface,
+    guint address_type,
+    guint access_control,
+    const GValue *access_control_param,
+    DBusGMethodInvocation *context)
 {
   SalutTubeStream *self = SALUT_TUBE_STREAM (iface);
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
   GError *error = NULL;
 
-  if (priv->state != SALUT_TUBE_CHANNEL_STATE_LOCAL_PENDING)
+  if (priv->state != TP_TUBE_CHANNEL_STATE_LOCAL_PENDING)
     {
       GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "Tube is not in the local pending state" };
@@ -2366,7 +2479,7 @@ salut_tube_stream_accept_stream_tube (SalutSvcChannelTypeStreamTube *iface,
     salut_muc_channel_send_presence (self->muc, NULL);
 #endif
 
-  salut_svc_channel_type_stream_tube_return_from_accept_stream_tube (context,
+  tp_svc_channel_type_stream_tube_return_from_accept (context,
       priv->address);
 }
 
@@ -2395,7 +2508,7 @@ salut_tube_stream_get_channel_type (TpSvcChannel *iface,
                                     DBusGMethodInvocation *context)
 {
   tp_svc_channel_return_from_get_channel_type (context,
-      SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+      TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
 }
 
 /**
@@ -2491,22 +2604,22 @@ salut_tube_stream_offer (SalutTubeStream *self,
 {
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
 
-  g_assert (priv->state == SALUT_TUBE_CHANNEL_STATE_NOT_OFFERED);
+  g_assert (priv->state == TP_TUBE_CHANNEL_STATE_NOT_OFFERED);
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     {
-      priv->state = SALUT_TUBE_CHANNEL_STATE_REMOTE_PENDING;
+      priv->state = TP_TUBE_CHANNEL_STATE_REMOTE_PENDING;
       salut_tubes_channel_send_iq_offer (priv->tubes_channel);
 
-      salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
-          self, SALUT_TUBE_CHANNEL_STATE_REMOTE_PENDING);
+      tp_svc_channel_interface_tube_emit_tube_channel_state_changed (
+          self, TP_TUBE_CHANNEL_STATE_REMOTE_PENDING);
     }
   else
     {
       /* muc tube is open as soon it's offered */
-      priv->state = SALUT_TUBE_CHANNEL_STATE_OPEN;
-      salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
-          self, SALUT_TUBE_CHANNEL_STATE_OPEN);
+      priv->state = TP_TUBE_CHANNEL_STATE_OPEN;
+      tp_svc_channel_interface_tube_emit_tube_channel_state_changed (
+          self, TP_TUBE_CHANNEL_STATE_OPEN);
       g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
     }
 
@@ -2538,13 +2651,13 @@ static void
 streamtube_iface_init (gpointer g_iface,
                        gpointer iface_data)
 {
-  SalutSvcChannelTypeStreamTubeClass *klass =
-      (SalutSvcChannelTypeStreamTubeClass *) g_iface;
+  TpSvcChannelTypeStreamTubeClass *klass =
+      (TpSvcChannelTypeStreamTubeClass *) g_iface;
 
-#define IMPLEMENT(x) salut_svc_channel_type_stream_tube_implement_##x (\
-    klass, salut_tube_stream_##x)
-  IMPLEMENT(offer_stream_tube);
-  IMPLEMENT(accept_stream_tube);
+#define IMPLEMENT(x, suffix) tp_svc_channel_type_stream_tube_implement_##x (\
+    klass, salut_tube_stream_##x##suffix)
+  IMPLEMENT(offer,_async);
+  IMPLEMENT(accept,_async);
 #undef IMPLEMENT
 }
 
